diff -ru ../sendmail-8.13.8-orig/sendmail/conf.c ././sendmail/conf.c
--- ../sendmail-8.13.8-orig/sendmail/conf.c	2006-03-22 17:49:33.000000000 -0500
+++ ././sendmail/conf.c	2006-11-12 10:01:19.000000000 -0500
@@ -6227,6 +6227,20 @@
 	/* runtime option for "MaxNOOPCommands" */
 	"_FFR_MAXNOOPCOMMANDS",
 #endif /* _FFR_MAXNOOPCOMMANDS */
+#if _FFR_ATTACK_SLEEP_LOG
+	/* log extra detail for every command over limit */
+	"_FFR_ATTACK_SLEEP_LOG",
+#endif /* _FFR_ATTACK_SLEEP_LOG */
+#if _FFR_ATTACK_FASTTERM
+	/* terminate connection after bad commands + _FFR_ATTACK_FASTTERM or *2
+	 * whichever is first */
+	"_FFR_ATTACK_FASTTERM",
+#endif /* _FFR_ATTACK_FASTTERM */
+#if _FFR_ATTACK_UPTO_LIMIT
+	/* allow upto, inclusively number of bad commands instead exclusively.
+	 * This allows you to set the MAX*CMDS to 1 */
+	"_FFR_ATTACK_UPTO_LIMIT",
+#endif /* _FFR_ATTACK_UPTO_LIMIT */
 #if _FFR_MAX_SLEEP_TIME
 	/* Limit sleep(2) time in libsm/clock.c */
 	"_FFR_MAX_SLEEP_TIME",
diff -ru ../sendmail-8.13.8-orig/sendmail/srvrsmtp.c ././sendmail/srvrsmtp.c
--- ../sendmail-8.13.8-orig/sendmail/srvrsmtp.c	2006-07-07 12:29:39.000000000 -0400
+++ ././sendmail/srvrsmtp.c	2006-11-12 09:52:14.000000000 -0500
@@ -3692,31 +3709,54 @@
 {
 	if (maxcount <= 0)	/* no limit */
 		return (time_t) 0;
-
+#if _FFR_ATTACK_UPTO_LIMIT
+	if (++(*pcounter) > maxcount)
+#else
 	if (++(*pcounter) >= maxcount)
+#endif /* _FFR_ATTACK_UPTO_LIMIT */
 	{
 		unsigned int shift;
 		time_t s;
 
+#if !_FFR_ATTACK_LOG_SLEEP
+# if _FFR_ATTACK_UPTO_LIMIT
+		if (*pcounter == maxcount+1 && LogLevel > 5)
+# else
 		if (*pcounter == maxcount && LogLevel > 5)
+# endif /* _FFR_ATTACK_UPTO_LIMIT */
 		{
 			sm_syslog(LOG_INFO, e->e_id,
 				  "%s: possible SMTP attack: command=%.40s, count=%u",
 				  CurSmtpClient, cname, *pcounter);
 		}
+#endif /*!_FFR_ATTACK_LOG_SLEEP */
 		shift = *pcounter - maxcount;
 		s = 1 << shift;
 		if (shift > MAXSHIFT || s >= MAXTIMEOUT || s <= 0)
 			s = MAXTIMEOUT;
+#if _FFR_ATTACK_FASTTERM
 
-#define IS_ATTACK(s)	((MaxChildren > 0 && *pcounter >= maxcount * 2)	\
+# define IS_ATTACK(s)	((MaxChildren > 0 && ((*pcounter >= maxcount + _FFR_ATTACK_FASTTERM) ||	\
+					     (*pcounter >= maxcount * 2)))			\
 				? STOP_ATTACK : (time_t) s)
-
+#else
+# define IS_ATTACK(s)	((MaxChildren > 0 && *pcounter >= maxcount * 2)	\
+				? STOP_ATTACK : (time_t) s)
+#endif /* _FFR_ATTACK_FASTTERM */
 		/* sleep at least 1 second before returning */
 		(void) sleep(*pcounter / maxcount);
 		s -= *pcounter / maxcount;
 		if (s >= MAXTIMEOUT || s < 0)
 			s = MAXTIMEOUT;
+#if _FFR_ATTACK_LOG_SLEEP
+		if (*pcounter >= maxcount && LogLevel > 5)
+		{
+			sm_syslog(LOG_INFO, e->e_id,
+				  "%s: possible SMTP attack: command=%.40s, count=%u, sleeptime=%d, return=%d",
+				  CurSmtpClient, cname, *pcounter, ((waitnow) ? s : 0 ) + (*pcounter / maxcount),
+				  (waitnow) ? IS_ATTACK(0) : IS_ATTACK(s));
+		}
+#endif /*_FFR_ATTACK_LOG_SLEEP */
 		if (waitnow && s > 0)
 		{
 			(void) sleep(s);
